@@ -20,12 +20,12 @@ B<baz>, B<bzr>, B<tla> (arch), B<darcs>.
 
 =over 4
 
-=item B<-c> B<--changelog> I<path>
+=item B<-c>, B<--changelog> I<path>
 
 Specify an alternate location for the changelog. By default debian/changelog is
 used.
 
-=item B<-r> B<--release>
+=item B<-r>, B<--release>
 
 Commit a release of the package. The version number is determined from
 debian/changelog, and is used to tag the package in the repository.
@@ -34,43 +34,43 @@ Note that svn/svk tagging conventions vary, so debcommit uses
 svnpath(1) to determine where the tag should be placed in the
 repository.
 
-=item B<-R> B<--release-use-changelog>
+=item B<-R>, B<--release-use-changelog>
 
 When used in conjunction with --release, if there are uncommited
 changes to the changelog then derive the commit message from those
 changes rather than using the default message.
 
-=item B<-m> I<text> B<--message> I<text>
+=item B<-m> I<text>, B<--message> I<text>
 
 Specify a commit message to use. Useful if the program cannot determine
 a commit message on its own based on debian/changelog, or if you want to
 override the default message.
 
-=item B<-n> B<--noact>
+=item B<-n>, B<--noact>
 
 Do not actually do anything, but do print the commands that would be run.
 
-=item B<-d> B<--diff>
+=item B<-d>, B<--diff>
 
 Instead of committing, do print the diff of what would have been committed if
 this option were not given. A typical usage scenario of this option is the
 generation of patches against the current working copy (e.g. when you don't have
 commit access right).
 
-=item B<-C> B<--confirm>
+=item B<-C>, B<--confirm>
 
 Display the generated commit message and ask for confirmation before committing
 it. It is also possible to edit the message at this stage; in this case, the
 confirmation prompt will be re-displayed after the editing has been performed.
 
-=item B<-e> B<--edit>
+=item B<-e>, B<--edit>
 
 Edit the generated commit message in your favorite editor before committing
 it.
 
-=item B<-a> B<--all>
+=item B<-a>, B<--all>
 
-Commit all files. This is the default operation when using a VCS other 
+Commit all files. This is the default operation when using a VCS other
 than git.
 
 =item I<files to commit>
@@ -78,10 +78,10 @@ than git.
 Specify which files to commit (debian/changelog is added to the list
 automatically.)
 
-=item B<-s> B<--strip-message>, B<--no-strip-message>
+=item B<-s>, B<--strip-message>, B<--no-strip-message>
 
-If this option is set and the commit message has been derived from the 
-changelog, the characters "* " will be stripped from the beginning of 
+If this option is set and the commit message has been derived from the
+changelog, the characters "* " will be stripped from the beginning of
 the message.
 
 This option is set by default and ignored if more than one line of
@@ -106,7 +106,7 @@ ignored for this purpose.  The currently recognised variables are:
 
 =item B<DEBCOMMIT_STRIP_MESSAGE>
 
-If this is set to I<no>, then it is the same as the --no-strip-message 
+If this is set to I<no>, then it is the same as the --no-strip-message
 command line parameter being used. The default is I<yes>.
 
 =item B<DEBCOMMIT_SIGN_TAGS>
@@ -336,6 +336,8 @@ if (@files_to_commit && !grep(/$changelog/,@files_to_commit)) {
     push @files_to_commit, $changelog;
 }
 
+# Main program
+
 my $prog=getprog();
 if (! defined $changelog) {
     die "debcommit: Could not find a Debian changelog\n";
@@ -383,6 +385,8 @@ if (not $confirm or confirm($message)) {
     tag($version) if $release;
 }
 
+# End of code, only subs below
+
 sub getprog {
     if (-d "debian") {
 	if (-d "debian/.svn") {
@@ -523,9 +527,9 @@ sub commit {
 	    : action($prog, "commit", "-m", $message, @files_to_commit);
     }
     elsif ($prog eq 'git') {
-	if (! @files_to_commit && $all) {
+	if (! @files_to_commit && ($all || $release)) {
 	    # check to see if the WC is clean. git-commit would exit
-	    # nonzero, so don't run it.
+	    # nonzero, so don't run it in --all or --release mode.
 	    my $status=`LANG=C git status`;
 	    if ($status=~/nothing to commit \(working directory clean\)/) {
 		    print $status;
@@ -588,13 +592,13 @@ sub commit {
 
 sub tag {
     my $tag=shift;
-    
+
     if ($prog eq 'svn' || $prog eq 'svk') {
 	my $svnpath=`svnpath`;
 	chomp $svnpath;
 	my $tagpath=`svnpath tags`;
 	chomp $tagpath;
-	
+
 	if (! action($prog, "copy", $svnpath, "$tagpath/$tag",
 		     "-m", "tagging version $tag")) {
 	    if (! action($prog, "mkdir", $tagpath,
@@ -624,7 +628,7 @@ sub tag {
 	} else {
 	    $subcommand="tag";
 	}
-	
+
 	if (! action($prog, $subcommand, $archpath, $tagpath)) {
 	    die "debcommit: failed tagging with $tag\n";
 	}
@@ -727,7 +731,7 @@ sub getmessage {
 	    next if /^\s*\[.*\]\s*$/; # maintainer name
 	    $ret .= $_;
 	}
-	
+
 	if (! length $ret) {
 	    if ($release) {
 		return;
@@ -834,6 +838,7 @@ sub git_repo_has_commits {
     system $command;
     return ($? >> 8 == 0) ? 1 : 0;
 }
+
 =head1 LICENSE
 
 This code is copyright by Joey Hess <joeyh@debian.org>, all rights reserved.
