@@ -264,7 +264,7 @@ while (@ARGV) {
     elsif ($ARGV[0] eq '--from') { $type = 'debs'; last; }
     elsif ($ARGV[0] =~ /^--w([plt])$/) { $wdiff_opt = "-$1"; shift; }
     elsif ($ARGV[0] =~ /^(--ignore-space|-w)$/) {
-	push @diff_opts, "-w"; 
+	push @diff_opts, "-w";
 	shift;
     }
     elsif ($ARGV[0] eq '--diffstat') { $show_diffstat = 1; shift; }
@@ -344,7 +344,7 @@ if ($type eq 'deb') {
 	my ($debc, $debI) = ('', '');
 	my %dpkg_env = ( LC_ALL => 'C' );
 	eval {
-	    fork_and_exec(exec => ['dpkg-deb', '-c', $deb],
+	    spawn(exec => ['dpkg-deb', '-c', $deb],
 		env => \%dpkg_env,
 		to_string => \$debc,
 		wait_child => 1);
@@ -354,7 +354,7 @@ if ($type eq 'deb') {
 	}
 
 	eval {
-	    fork_and_exec(exec => ['dpkg-deb', '-I', $deb],
+	    spawn(exec => ['dpkg-deb', '-I', $deb],
 		env => \%dpkg_env,
 		to_string => \$debI,
 		wait_child => 1);
@@ -420,7 +420,7 @@ elsif ($type eq 'changes' or $type eq 'debs') {
 	    my ($debc, $debI) = ('', '');
 	    my %dpkg_env = ( LC_ALL => 'C' );
 	    eval {
-		fork_and_exec(exec => ['dpkg-deb', '-c', $deb],
+		spawn(exec => ['dpkg-deb', '-c', $deb],
 		    to_string => \$debc,
 		    env => \%dpkg_env,
 		    wait_child => 1);
@@ -429,7 +429,7 @@ elsif ($type eq 'changes' or $type eq 'debs') {
 		fatal "dpkg-deb -c $deb failed!";
 	    }
 	    eval {
-		fork_and_exec(exec => ['dpkg-deb', '-I', $deb],
+		spawn(exec => ['dpkg-deb', '-I', $deb],
 		    to_string => \$debI,
 		    env => \%dpkg_env,
 		    wait_child => 1);
@@ -499,7 +499,7 @@ elsif ($type eq 'dsc') {
 		if ($file =~ /\.diff\.gz$/) {
 		    $diffs[$i] = cwd() . '/' . $file;
 		}
-		elsif ($file =~ /(\.orig)?\.tar\.(gz|bz2|lzma)$/) {
+		elsif ($file =~ /(\.orig)?\.tar\.(gz|bz2|lzma|xz)$/) {
 		    $origs[$i] = $file;
 		}
 	    } else {
@@ -543,7 +543,7 @@ elsif ($type eq 'dsc') {
 	and scalar(@excludes) == 0 and $use_interdiff and !$wdiff_source_control) {
 	# same orig tar ball, interdiff exists and not wdiffing
 
-	fork_and_exec(exec => ['interdiff', '-z', @diff_opts, $diffs[1], $diffs[2]],
+	spawn(exec => ['interdiff', '-z', @diff_opts, $diffs[1], $diffs[2]],
 	      to_file => $filename,
 	      wait_child => 1);
 	if ($have_diffstat and $show_diffstat) {
@@ -551,7 +551,7 @@ elsif ($type eq 'dsc') {
 			    . " " . basename($diffs[2]) . "\n\n";
 	    $header =~ s/\.diff\.gz//g;
 	    print $header;
-	    fork_and_exec(exec => ['diffstat', $filename],
+	    spawn(exec => ['diffstat', $filename],
 		wait_child => 1);
 	    print "\n";
 	}
@@ -582,7 +582,7 @@ elsif ($type eq 'dsc') {
 	    push (@opts, '--skip-patches') if $dscformats[$i] eq '3.0 (quilt)';
 	    my $diri = ${"dir$i"};
 	    eval {
-		fork_and_exec(exec => ['dpkg-source', @opts, $dscs[$i]],
+		spawn(exec => ['dpkg-source', @opts, $dscs[$i]],
 		      to_file => '/dev/null',
 		      chdir => $diri,
 		      wait_child => 1);
@@ -594,7 +594,7 @@ elsif ($type eq 'dsc') {
 		my $dscx = basename $dscs[$i];
 		cp $diffs[$i], $diri || fatal "copy $diffs[$i] $diri: $!";
 		cp $dscs[$i], $diri || fatal "copy $dscs[$i] $diri: $!";
-		fork_and_exec(exec => ['dpkg-source', @opts, $dscx],
+		spawn(exec => ['dpkg-source', @opts, $dscx],
 		      to_file => '/dev/null',
 		      chdir => $diri,
 		      wait_child => 1);
@@ -633,7 +633,7 @@ elsif ($type eq 'dsc') {
 		    }
 
 		    if ($found) {
-			fork_and_exec(exec => ['tar', "--$comp", '-xf', $_],
+			spawn(exec => ['tar', "--$comp", '-xf', $_],
 			    to_file => '/dev/null',
 			    wait_child => 1,
 			    chdir => "$diri/$sdiri",
@@ -654,11 +654,11 @@ elsif ($type eq 'dsc') {
 
 	# Execute diff and remove the common prefixes $dir1/$dir2, so the patch can be used with -p1,
 	# as if when interdiff would have been used:
-	fork_and_exec(exec => \@command, to_file => $filename, wait_child => 1, nocheck => 1);
+	spawn(exec => \@command, to_file => $filename, wait_child => 1, nocheck => 1);
 
 	if ($have_diffstat and $show_diffstat) {
 	    print "diffstat for $sdir1 $sdir2\n\n";
-	    fork_and_exec(exec => ['diffstat', $filename],
+	    spawn(exec => ['diffstat', $filename],
 		wait_child => 1);
 	    print "\n";
 	}
@@ -752,7 +752,7 @@ if ($show_moved and $type ne 'deb') {
 	print join("\n",@deblosses), "\n\n";
     }
 
-    # We start by determining which files are in the first set of debs, the 
+    # We start by determining which files are in the first set of debs, the
     # second set of debs or both.
     my %files;
     grep $files{$_}--, @D1;
@@ -794,7 +794,7 @@ if ($show_moved and $type ne 'deb') {
 	# list it separately.
 	my @fdebs1 = split (/:/, $files1{$file});
 	my @fdebs2 = split (/:/, $files2{$file});
-	
+
 	if (@fdebs1 == 1 && @fdebs2 == 1) {
 	    push @{$changes{$fdebs1[0]}{$fdebs2[0]}}, $file;
 	} else {
@@ -911,7 +911,7 @@ for my $debname (@CommonDebs) {
 	my $debpath = "${\"DebPaths$i\"}{$debname}";
 	my $diri = ${"dir$i"};
 	eval {
-	    fork_and_exec(exec => ['dpkg-deb', '-e', $debpath, $diri],
+	    spawn(exec => ['dpkg-deb', '-e', $debpath, $diri],
 		wait_child => 1);
 	};
 	if ($@) {
@@ -1030,7 +1030,7 @@ sub wdiff_control_files($$$$$)
 	push @opts, $wdiff_opt if $wdiff_opt;
 	my $wdiff = '';
 	eval {
-	    fork_and_exec(exec => ['wdiff', @opts, "$dir1/$cf", "$dir2/$cf"],
+	    spawn(exec => ['wdiff', @opts, "$dir1/$cf", "$dir2/$cf"],
 		to_string => \$wdiff,
 		wait_child => 1);
 	};
@@ -1047,6 +1047,7 @@ sub wdiff_control_files($$$$$)
 		my $msg = ucfirst($cf) . " files$usepkgname: wdiff output";
 		print $msg, "\n", '-' x length $msg, "\n";
 		print $wdiff;
+		$status = 1;
 	    } else {
 		my @output;
 		@output = split /\n/, $wdiff;
@@ -1054,8 +1055,8 @@ sub wdiff_control_files($$$$$)
 		my $msg = ucfirst($cf) . " files$usepkgname: lines which differ (wdiff format)";
 		print "\n", $msg, "\n", '-' x length $msg, "\n";
                 print join("\n",@output), "\n";
+		$status = 1;
 	    }
-	    $status = 1;
 	}
     }
 
