@@ -1,10 +1,10 @@
-#! /usr/bin/perl -w
+#!/usr/bin/perl
 
 # Perl version of Christoph Lameter's build program, renamed debuild.
 # Written by Julian Gilbey, December 1998.
 
 # Copyright 1999-2003, Julian Gilbey <jdg@debian.org>
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -33,7 +33,7 @@
 # then to run lintian on the resulting .changes file.
 # Lintian options may be specified after --lintian-opts; all following
 # options will be passed only to lintian.
-# 
+#
 # As this may be running setuid, we make sure to clean out the
 # environment before we perform the build, subject to any -e etc.
 # options.  Also wise for building the packages, anyway.
@@ -48,6 +48,7 @@
 # Please file a bug report if this is the case!
 
 use strict;
+use warnings;
 use 5.008;
 use File::Basename;
 use filetest 'access';
@@ -118,7 +119,7 @@ Second usage method:
         --no-lintian        Do not run lintian
         --[no-]tgz-check    Do [not] check for an .orig.tar.gz before running
                             dpkg-buildpackage if we have a Debian revision
-                            (Default: check) 
+                            (Default: check)
         --username          Run debrsign instead of debsign, using the
                             supplied credentials
 
@@ -274,7 +275,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     foreach my $var ($dpkg_opts_var, $lintian_opts_var) {
 	$shell_cmd .= "eval set -- \$$var;\n";
 	$shell_cmd .= "echo \">>> $var BEGIN <<<\";\n";
-	$shell_cmd .= 'while [ $# -gt 0 ]; do printf "%s\n" $1; shift; done;' . "\n";
+	$shell_cmd .= 'while [ $# -gt 0 ]; do printf "%s\n" "$1"; shift; done;' . "\n";
 	$shell_cmd .= "echo \">>> $var END <<<\";\n";
     }
     # Not totally efficient, but never mind
@@ -614,8 +615,8 @@ unless ($preserve_env) {
 umask 022;
 
 # Start by duping STDOUT and STDERR
-open OLDOUT, ">&STDOUT" or fatal "can't dup stdout: $!\n";
-open OLDERR, ">&STDERR" or fatal "can't dup stderr: $!\n";
+open OLDOUT, ">&", \*STDOUT or fatal "can't dup stdout: $!\n";
+open OLDERR, ">&", \*STDERR or fatal "can't dup stderr: $!\n";
 
 # Look for the debian changelog
 my $chdir = 0;
@@ -660,7 +661,7 @@ if ($check_dirname_level ==  2 or
     my $gooddir;
     if ($re =~ m%/%) { $gooddir = eval "cwd() =~ /^$re\$/;"; }
     else { $gooddir = eval "basename(cwd()) =~ /^$re\$/;"; }
-    
+
     if (! $gooddir) {
 	my $pwd = cwd();
 	die <<"EOF";
@@ -743,7 +744,7 @@ if ($command_version eq 'dpkg') {
 			     final-clean)) {
 	    if ($hook{$hookname}) { push @skip_hooks, $hookname; }
 	}
-	if (@skip_hooks) {	
+	if (@skip_hooks) {
 	    $emulate_dpkgbp = 1;
 	    warn "$progname: emulating dpkg-buildpackage as the following hooks were defined:\n"
 		. "  " . join(", ", @skip_hooks) . "\n\n";
@@ -786,6 +787,8 @@ if ($command_version eq 'dpkg') {
     my $compression='';
     my $comp_level='';
 
+    my $dirn = basename(cwd());
+
     # and one for us
     my @debsign_opts = ();
     # and one for dpkg-cross if needed
@@ -813,13 +816,15 @@ if ($command_version eq 'dpkg') {
 	/^-Z/ and $compression=$_, push(@dpkg_opts, $_), next;
 	/^-z/ and $comp_level=$_, push(@dpkg_opts, $_), next;
 	$_ eq '-tc' and $cleansource=1, push(@dpkg_opts, $_), next;
-	/^-t(.*)/ and $targetgnusystem=$1, push(@dpkg_opts, $_), next; # Ditto	
-	$_ eq '-nc' and $noclean=1, $binaryonly ||= '-b', push(@dpkg_opts, $_),
+	/^-t(.*)/ and $targetgnusystem=$1, push(@dpkg_opts, $_), next; # Ditto
+	$_ eq '-nc' and $noclean=1, push(@dpkg_opts, $_),
 	    next;
 	$_ eq '-b' and $binaryonly=$_, $binarytarget='binary',
 	    push(@dpkg_opts, $_), next;
 	$_ eq '-B' and $binaryonly=$_, $binarytarget='binary-arch',
 	    push(@dpkg_opts, $_), next;
+	$_ eq '-A' and $binaryonly=$_, $binarytarget='binary-indep',
+	    push(@dpkg_opts, $_), next;
 	$_ eq '-S' and $sourceonly=$_, push(@dpkg_opts, $_), next;
 	    # Explained below, no implied -d
 	/^-v(.*)/ and $since=$1, push(@dpkg_opts, $_), next;
@@ -864,12 +869,14 @@ if ($command_version eq 'dpkg') {
 	/^-z/ and $comp_level=$_, push(@dpkg_opts, $_), next;
 	$_ eq '-tc' and $cleansource=1, push(@dpkg_opts, $_), next;
 	/^-t(.*)/ and $targetgnusystem=$1, $checkbuilddep=0, next;
-	$_ eq '-nc' and $noclean=1, $binaryonly ||= '-b', push(@dpkg_opts, $_),
+	$_ eq '-nc' and $noclean=1, push(@dpkg_opts, $_),
 	    next;
 	$_ eq '-b' and $binaryonly=$_, $binarytarget='binary',
 	    push(@dpkg_opts, $_), next;
 	$_ eq '-B' and $binaryonly=$_, $binarytarget='binary-arch',
 	    push(@dpkg_opts, $_), next;
+	$_ eq '-A' and $binaryonly=$_, $binarytarget='binary-indep',
+	    push(@dpkg_opts, $_), next;
 	$_ eq '-S' and $sourceonly=$_, $checkbuilddep=0, push(@dpkg_opts, $_),
 	    next;
 	/^-v(.*)/ and $since=$1, push(@dpkg_opts, $_), next;
@@ -1070,6 +1077,10 @@ if ($command_version eq 'dpkg') {
 	    }
 	}
 
+	chdir '..' or fatal "can't chdir ..: $!";
+	system_withecho('dpkg-source', '--before-build', $dirn);
+	chdir $dirn or fatal "can't chdir $dirn: $!";
+
 	# First dpkg-buildpackage action: run dpkg-checkbuilddeps
 	if ($checkbuilddep) {
 	    if ($binarytarget eq 'binary-arch') {
@@ -1103,7 +1114,6 @@ EOT
 
 	# Next dpkg-buildpackage action: dpkg-source
 	if (! $binaryonly) {
-	    my $dirn = basename(cwd());
 	    my @cmd = (qw(dpkg-source));
 	    push @cmd, @passopts;
 	    push @cmd, $diffignore if $diffignore;
@@ -1121,7 +1131,7 @@ EOT
 	# Next dpkg-buildpackage action: build and binary targets
 	if (! $sourceonly) {
 	    system_withecho('debian/rules', 'build');
-	
+
 	    run_hook('binary', 1);
 
 	    if ($< == 0) {
@@ -1150,7 +1160,7 @@ EOT
 	push @cmd, "-v$since" if $since;
 	push @cmd, "-C$desc" if $desc;
 	print STDERR " ", join(" ", @cmd), "\n";
-	
+
 	open GENCHANGES, "-|", @cmd or fatal "can't exec dpkg-genchanges: $!";
 	my @changefilecontents;
 	@changefilecontents = <GENCHANGES>;
@@ -1162,11 +1172,11 @@ EOT
 	close CHANGES
 	    or fatal "problem writing to ../$changes: $!";
 
-	# check Ubuntu merge Policy: When merging with Debian, -v must be used and the
-	# remaining changes described
+	# check Ubuntu merge Policy: When merging with Debian, -v must be used
+	# and the remaining changes described
 	my $ch = join "\n", @changefilecontents;
 	if ($sourceonly && $version =~ /ubuntu1$/ && $ENV{'DEBEMAIL'} =~ /ubuntu/ &&
-	    $ch =~ /(merge|sync).*Debian/i) { 
+	    $ch =~ /(merge|sync).*Debian/i) {
 	    push (@warnings, "Ubuntu merge policy: when merging Ubuntu packages with Debian, -v must be used") unless $since;
 	    push (@warnings, "Ubuntu merge policy: when merging Ubuntu packages with Debian, changelog must describe the remaining Ubuntu changes")
 		unless $ch =~ /Changes:.*(remaining|Ubuntu)(.|\n )*(differen|changes)/is;
@@ -1183,6 +1193,9 @@ EOT
 	    }
 	}
 
+	chdir '..' or fatal "can't chdir ..: $!";
+	system_withecho('dpkg-source', '--after-build', $dirn);
+	chdir $dirn or fatal "can't chdir $dirn: $!";
 
 	# identify the files listed in $changes; this will be used for the
 	# emulation of the dpkg-buildpackage fileomitted() function
@@ -1247,7 +1260,7 @@ EOT
     if ($signchanges) {
     foreach my $var (keys %store_vars) {
         $ENV{$var} = $store_vars{$var};
-    }  
+    }
 	print "Now signing changes and any dsc files...\n";
 	if ($username) {
 	    system('debrsign', @debsign_opts, $username, $changes) == 0
@@ -1284,8 +1297,8 @@ EOT
     close STDOUT;
     close STDERR;
     close BUILD;
-    open STDOUT, ">&OLDOUT";
-    open STDERR, ">&OLDERR";
+    open STDOUT, ">&", \*OLDOUT;
+    open STDERR, ">&", \*OLDERR;
     exit 0;
 }
 else {
@@ -1371,8 +1384,8 @@ sub fatal($) {
 	close STDOUT;
 	close STDERR;
 	close BUILD;
-	open STDOUT, ">&OLDOUT";
-	open STDERR, ">&OLDERR";
+	open STDOUT, ">&", \*OLDOUT;
+	open STDERR, ">&", \*OLDERR;
     }
     die $msg;
 }
