@@ -27,6 +27,7 @@
 # Later modifications: see debian/changelog
 
 use strict;
+use Cwd qw(abs_path);
 use IO::Dir;
 use IO::File;
 use Digest::MD5;
@@ -103,9 +104,14 @@ sub wget {
     my ($file, $url) = @_;
 
     # schemes not supported by all backends
-    if ($url =~ m!^(file|copy)://(/.+)!) {
-	if ($1 eq "copy" or not link($2, $file)) {
-	    system ('cp', '-a', $2, $file);
+    if ($url =~ m!^(file|copy):(.+)!) {
+	my $path = abs_path($2);
+	unless ($path) {
+	    warn "$progname: unable to resolve full path for $2: $!\n";
+	    return 1;
+	}
+	if ($1 eq "copy" or not link($path, $file)) {
+	    system 'cp', '-aL', $path, $file;
 	    return $? >> 8;
 	}
 	return;
@@ -165,8 +171,8 @@ sub get_file {
 	foreach my $path (@dget_path) {
 	    next unless -e "$path/$file";
 
+	    my $fh5 = new IO::File("$path/$file") or next;
 	    my $md5 = Digest::MD5->new;
-	    my $fh5 = new IO::File("$path/$file") or die "$path/$file: $!";
 	    my $md5sum_new = Digest::MD5->new->addfile($fh5)->hexdigest();
 	    close $fh5;
 
@@ -175,7 +181,7 @@ sub get_file {
 		    print "$progname: using $path/$file (hardlink)\n" unless $opt->{'quiet'};
 		} else {
 		    print "$progname: using $path/$file (copy)\n" unless $opt->{'quiet'};
-		    system ('cp', '-a', "$path/$file", $file);
+		    system 'cp', '-aL', "$path/$file", $file;
 		}
 		last;
 	    }
@@ -192,9 +198,10 @@ sub get_file {
     }
 
     # try apt-get if it is still not there
-    if (not -e $file and $file =~ m!^([a-z0-9][a-z0-9.+-]+)_[^/]+\.(?:diff\.gz|tar\.gz)$!) {
-	my $cmd = "apt-get source --print-uris $1";
-	my $apt = new IO::File("$cmd |") or die "$cmd: $!";
+    if (not -e $file and $file =~ m!^([a-z0-9.+-]{2,})_[^/]+\.(?:diff\.gz|tar\.gz)$!) {
+	my @cmd = ('apt-get', 'source', '--print-uris', $1);
+	my $cmd = join ' ', @cmd;
+	open(my $apt, '-|', @cmd) or die "$cmd: $!";
 	while(<$apt>) {
 	    if (/'(\S+)'\s+\S+\s+\d+\s+([\da-f]+)/i and $2 eq $md5sum) {
 		if (wget($file, $1)) {
@@ -265,7 +272,9 @@ sub apt_get {
 	if ($qversion and /^ [ *]{3} ($qversion) 0/) {
 	    while (<$apt>) {
 		last OUTER unless /^  *(?:\d+) (\S+)/;
-		push @hosts, $1;
+		(my $host = $1) =~ s@/$@@;
+		next if $host eq '/var/lib/dpkg/status';
+		push @hosts, $host;
 	    }
 	}
     }
@@ -277,7 +286,7 @@ sub apt_get {
 	die "$progname: no hostnames in apt-cache policy $package for $version found\n";
     }
 
-    $apt = new IO::File("LC_ALL=C apt-cache --all-versions show $package |")
+    $apt = new IO::File("LC_ALL=C apt-cache show $package=$version |")
 	or die "$!";
     my ($v, $p, $filename, $md5sum);
     while (<$apt>) {
@@ -315,7 +324,7 @@ sub apt_get {
     if (-f "/etc/apt/sources.list") {
 	$apt = new IO::File("/etc/apt/sources.list") or die "/etc/apt/sources.list: $!";
 	while (<$apt>) {
-	    if (/^\s*deb\s*($host_re\S+)/) {
+	    if (/^\s*deb\s*($host_re\b)/) {
 		push @repositories, $1;
 	    }
 	}
@@ -328,7 +337,7 @@ sub apt_get {
 	$_ = "/etc/apt/sources.list.d/$_";
 	$apt = new IO::File("$_") or die "$_: $!";
 	while (<$apt>) {
-	    if (/^\s*deb\s*($host_re\S+)/) {
+	    if (/^\s*deb\s*($host_re\b)/) {
 		push @repositories, $1;
 	    }
 	}
@@ -442,7 +451,7 @@ for my $arg (@ARGV) {
 	    if ($dget_verify) { # We are duplicating work here a bit as
 		# dpkg-source -x will also verify signatures. Still, we
 		# also want to barf with -d, and on unsigned packages.
-		system ('dscverify', $found_dsc);
+		system 'dscverify', $found_dsc;
 		exit $? >> 8 if $? >> 8 != 0;
 	    }
 	    my @cmd = qw(dpkg-source -x);
@@ -487,7 +496,7 @@ dget -- Download Debian source and binary packages
 
 =item B<dget> [I<options>] I<URL> ...
 
-=item B<dget> [I<options>] I<package>[=I<version>]
+=item B<dget> [I<options>] I<package>[B<=>I<version>]
 
 =back
 
@@ -497,7 +506,8 @@ B<dget> downloads Debian packages.  In the first form, B<dget> fetches
 the requested URLs.  If this is a .dsc or .changes file, then B<dget>
 acts as a source-package aware form of B<wget>: it also fetches any
 files referenced in the .dsc/.changes file.  The downloaded source is
-then unpacked by B<dpkg-source>.
+then checked with B<dscverify> and, if successful, unpacked by
+B<dpkg-source>.
 
 In the second form, B<dget> downloads a I<binary> package (i.e., a
 I<.deb> file) from the Debian mirror configured in
@@ -561,13 +571,13 @@ using B<dscverify>.
 
 Run B<dpkg-buildpackage -b -uc> on the downloaded source package.
 
-=item B<--path> DIR[:DIR...]
+=item B<--path> I<DIR>[B<:>I<DIR> ...]
 
 In addition to I</var/cache/apt/archives>, B<dget> uses the
 colon-separated list given as argument to B<--path> to find files with
 a matching md5sum.  For example: "--path
 /srv/pbuilder/result:/home/cb/UploadQueue".  If DIR is empty (i.e.,
-"S<--path ''>" is specified), then any previously listed directories
+"--path ''" is specified), then any previously listed directories
 or directories specified in the configuration files will be ignored.
 This option may be specified multiple times, and all of the
 directories listed will be searched; hence, the above example could
@@ -602,19 +612,19 @@ ignored for this purpose.  The currently recognised variable is:
 
 =over 4
 
-=item DGET_PATH
+=item B<DGET_PATH>
 
 This can be set to a colon-separated list of directories in which to
 search for files in addition to the default
 I</var/cache/apt/archives>.  It has the same effect as the B<--path>
 command line option.  It is not set by default.
 
-=item DGET_UNPACK
+=item B<DGET_UNPACK>
 
 Set to 'no' to disable extracting downloaded source packages.  Default
 is 'yes'.
 
-=item DGET_VERIFY
+=item B<DGET_VERIFY>
 
 Set to 'no' to disable checking signatures of downloaded source
 packages.  Default is 'yes'.
@@ -638,4 +648,5 @@ of the License, or (at your option) any later version.
 
 =head1 SEE ALSO
 
-B<apt-get>(1), B<debdiff>(1), B<dpkg-source>(1), B<curl>(1), B<wget>(1).
+B<apt-get>(1), B<debcheckout>(1), B<debdiff>(1), B<dpkg-source>(1), B<curl>(1),
+B<wget>(1).
